@@ -35,11 +35,10 @@
 #define CONNMAN_API_SUBJECT_TO_CHANGE
 #include <connman/plugin.h>
 #include <connman/device.h>
+#include <connman/inet.h>
 #include <connman/rtnl.h>
 #include <connman/log.h>
 
-#include "inet.h"
-
 struct ethernet_data {
 	int index;
 	unsigned flags;
@@ -118,7 +117,7 @@ static int ethernet_enable(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	return inet_ifup(ethernet->index);
+	return connman_inet_ifup(ethernet->index);
 }
 
 static int ethernet_disable(struct connman_device *device)
@@ -127,7 +126,31 @@ static int ethernet_disable(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	return inet_ifdown(ethernet->index);
+	return connman_inet_ifdown(ethernet->index);
+}
+
+static int ethernet_connect(struct connman_device *device)
+{
+	struct ethernet_data *ethernet = connman_device_get_data(device);
+
+	DBG("device %p", device);
+
+	if (!(ethernet->flags & IFF_LOWER_UP))
+		return -ENOTCONN;
+
+	return connman_device_set_connected(device, TRUE);
+}
+
+static int ethernet_disconnect(struct connman_device *device)
+{
+	struct ethernet_data *ethernet = connman_device_get_data(device);
+
+	DBG("device %p", device);
+
+	if (!(ethernet->flags & IFF_LOWER_UP))
+		return -ENOTCONN;
+
+	return connman_device_set_connected(device, FALSE);
 }
 
 static struct connman_device_driver ethernet_driver = {
@@ -137,6 +160,8 @@ static struct connman_device_driver ethernet_driver = {
 	.remove		= ethernet_remove,
 	.enable		= ethernet_enable,
 	.disable	= ethernet_disable,
+	.connect	= ethernet_connect,
+	.disconnect	= ethernet_disconnect,
 };
 
 static int ethernet_init(void)
