@@ -64,7 +64,7 @@ static void append_path(gpointer value, gpointer user_data)
 	struct connman_service *service = value;
 	DBusMessageIter *iter = user_data;
 
-	if (service->path == NULL)
+	if (service->path == NULL || service->hidden == TRUE)
 		return;
 
 	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
@@ -210,6 +210,26 @@ static enum connman_service_error string2error(const char *error)
 	return CONNMAN_SERVICE_ERROR_UNKNOWN;
 }
 
+const char *__connman_service_default(void)
+{
+	struct connman_service *service;
+	GSequenceIter *iter;
+
+	iter = g_sequence_get_begin_iter(service_list);
+
+	if (g_sequence_iter_is_end(iter) == TRUE)
+		return "";
+
+	service = g_sequence_get(iter);
+	if (service == NULL)
+		return "";
+
+	if (service->state != CONNMAN_SERVICE_STATE_READY)
+		return "";
+
+	return type2string(service->type);
+}
+
 static void state_changed(struct connman_service *service)
 {
 	DBusMessage *signal;
@@ -963,6 +983,30 @@ int __connman_service_set_carrier(struct connman_service *service,
 	return connman_service_set_favorite(service, carrier);
 }
 
+static void default_changed(void)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "DefaultTechnology";
+	const char *str = __connman_service_default();
+
+	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_STRING_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_STRING, &str);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(connection, signal);
+}
+
 int __connman_service_indicate_state(struct connman_service *service,
 					enum connman_service_state state)
 {
@@ -979,6 +1023,9 @@ int __connman_service_indicate_state(struct connman_service *service,
 	if (service->state == state)
 		return -EALREADY;
 
+	if (service->state == CONNMAN_SERVICE_STATE_READY)
+		default_changed();
+
 	if (service->state == CONNMAN_SERVICE_STATE_FAILURE &&
 				state == CONNMAN_SERVICE_STATE_IDLE)
 		return -EINVAL;
@@ -1046,6 +1093,8 @@ int __connman_service_indicate_default(struct connman_service *service)
 {
 	DBG("service %p", service);
 
+	default_changed();
+
 	return 0;
 }
 
@@ -1414,30 +1463,6 @@ struct connman_service *__connman_service_lookup_from_device(struct connman_devi
 	return service;
 }
 
-static enum connman_service_type convert_device_type(struct connman_device *device)
-{
-	enum connman_device_type type = connman_device_get_type(device);
-
-	switch (type) {
-	case CONNMAN_DEVICE_TYPE_UNKNOWN:
-	case CONNMAN_DEVICE_TYPE_VENDOR:
-	case CONNMAN_DEVICE_TYPE_WIFI:
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_MBM:
-	case CONNMAN_DEVICE_TYPE_HSO:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
-		break;
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
-		return CONNMAN_SERVICE_TYPE_ETHERNET;
-	}
-
-	return CONNMAN_SERVICE_TYPE_UNKNOWN;
-}
-
 /**
  * __connman_service_create_from_device:
  * @device: device structure
@@ -1467,7 +1492,7 @@ struct connman_service *__connman_service_create_from_device(struct connman_devi
 		return service;
 	}
 
-	service->type = convert_device_type(device);
+	service->type = __connman_device_get_service_type(device);
 
 	service->device = device;
 
